@@ -7,7 +7,6 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-import re
 from abc import ABCMeta
 from copy import copy
 from itertools import count
@@ -18,12 +17,13 @@ from elementpath.etree import etree_tostring
 
 from .exceptions import XMLSchemaAttributeError, XMLSchemaTypeError, XMLSchemaValueError
 from .aliases import ElementType, XMLSourceType, NamespacesType, BaseXsdType, DecodeType
-from .helpers import get_namespace, get_prefixed_qname, local_name, raw_xml_encode
+from .helpers import get_namespace, get_prefixed_qname, local_name, update_namespaces, \
+    raw_xml_encode, get_namespace_map
 from .converters import ElementData, XMLSchemaConverter
 from .resources import XMLResource
 from . import validators
 
-if TYPE_CHECKING:  # pragma: no cover
+if TYPE_CHECKING:
     from .validators import XMLSchemaValidationError, XsdElement
 
 
@@ -46,6 +46,7 @@ class DataElement(MutableSequence['DataElement']):
 
     value: Optional[Any] = None
     tail: Optional[str] = None
+    xmlns: Optional[List[Tuple[str, str]]] = None
     xsd_element: Optional['XsdElement'] = None
     xsd_type: Optional[BaseXsdType] = None
     _encoder: Optional['XsdElement'] = None
@@ -54,6 +55,7 @@ class DataElement(MutableSequence['DataElement']):
                  value: Optional[Any] = None,
                  attrib: Optional[Dict[str, Any]] = None,
                  nsmap: Optional[MutableMapping[str, str]] = None,
+                 xmlns: Optional[List[Tuple[str, str]]] = None,
                  xsd_element: Optional['XsdElement'] = None,
                  xsd_type: Optional[BaseXsdType] = None) -> None:
 
@@ -69,6 +71,8 @@ class DataElement(MutableSequence['DataElement']):
             self.attrib.update(attrib)
         if nsmap is not None:
             self.nsmap.update(nsmap)
+        if xmlns is not None:
+            self.xmlns = xmlns
 
         if xsd_element is not None:
             self.xsd_element = xsd_element
@@ -185,6 +189,12 @@ class DataElement(MutableSequence['DataElement']):
         """The prefixed name, or the tag if no prefix is defined for its namespace."""
         return get_prefixed_qname(self.tag, self.nsmap)
 
+    @property
+    def display_name(self) -> str:
+        """The prefixed name, or the tag if it's associated with the default namespace."""
+        prefixed_name = self.prefixed_name
+        return self.name if ':' not in prefixed_name else prefixed_name
+
     @property
     def local_name(self) -> str:
         """The local part of the tag."""
@@ -214,44 +224,24 @@ class DataElement(MutableSequence['DataElement']):
             if tag is None or tag == child.tag:
                 yield child
 
-    def get_namespaces(self, namespaces: Optional[NamespacesType] = None) -> NamespacesType:
+    def get_namespaces(self, namespaces: Optional[NamespacesType] = None,
+                       root_only: bool = True) -> NamespacesType:
         """
-        Returns an overall namespace map for DetaElement and its descendants,
-        resolving prefix redefinitions.
+        Returns an overall namespace map for DetaElement, resolving prefix redefinitions.
 
         :param namespaces: builds the namespace map starting over the dictionary provided.
+        :param root_only: if `True` processes only the namespaces declared in the data \
+        element, otherwise precesses also other namespaces declared in its descendants.
         """
         namespaces = copy(namespaces) if namespaces is not None else {}
-        nsmap = None
-
-        for elem in self.iter():
-            if nsmap is elem.nsmap:
-                continue
-            else:
-                nsmap = elem.nsmap
-                for prefix, uri in nsmap.items():
-                    if not prefix:
-                        if not uri:
-                            continue
-                        elif '' not in namespaces:
-                            if self.namespace:
-                                namespaces[prefix] = uri
-                                continue
-                        elif namespaces[''] == uri:
-                            continue
-                        prefix = 'default'
-
-                    while prefix in namespaces:
-                        if namespaces[prefix] == uri:
-                            break
-                        match = re.search(r'(\d+)$', prefix)
-                        if match:
-                            index = int(match.group()) + 1
-                            prefix = prefix[:match.span()[0]] + str(index)
-                        else:
-                            prefix += '0'
-                    else:
-                        namespaces[prefix] = uri
+        if root_only:
+            update_namespaces(namespaces, self.nsmap.items(), root_declarations=True)
+        else:
+            nsmap = None
+            for elem in self.iter():
+                if nsmap is not elem.nsmap:
+                    nsmap = elem.nsmap
+                    update_namespaces(namespaces, nsmap.items(), elem is self)
 
         return namespaces
 
@@ -295,7 +285,7 @@ class DataElement(MutableSequence['DataElement']):
             raise XMLSchemaValueError("%r has no schema bindings" % self)
 
         kwargs: Dict[str, Any] = {
-            'namespaces': self.get_namespaces(namespaces),
+            'namespaces': self.get_namespaces(namespaces, root_only=False),
             'converter': DataElementConverter,
             'use_defaults': use_defaults,
         }
@@ -322,7 +312,7 @@ class DataElement(MutableSequence['DataElement']):
         :raises: :exc:`XMLSchemaValidationError` if the object is invalid \
         and ``validation='strict'``.
         """
-        kwargs['namespaces'] = self.get_namespaces(kwargs.get('namespaces'))
+        kwargs['namespaces'] = self.get_namespaces(kwargs.get('namespaces'), False)
         if 'converter' not in kwargs:
             kwargs['converter'] = DataElementConverter
 
@@ -348,7 +338,7 @@ class DataElement(MutableSequence['DataElement']):
         :param namespaces: is an optional mapping from namespace prefix to URI. \
         Provided namespaces are registered before serialization. Ignored if the \
         provided *elem* argument is a lxml Element instance.
-        :param indent: the base line indentation.
+        :param indent: the baseline indentation.
         :param max_lines: if truncate serialization after a number of lines \
         (default: do not truncate).
         :param spaces_for_tab: number of spaces for replacing tab characters. For \
@@ -361,7 +351,7 @@ class DataElement(MutableSequence['DataElement']):
         """
         root, _ = self.encode(validation='lax')
         if not hasattr(root, 'nsmap'):
-            namespaces = self.get_namespaces(namespaces)
+            namespaces = self.get_namespaces(namespaces, root_only=False)
 
         _string = etree_tostring(
             elem=root,
@@ -486,6 +476,28 @@ class DataElementConverter(XMLSchemaConverter):
         kwargs.update(attr_prefix='', text_key='', cdata_prefix='')
         super(DataElementConverter, self).__init__(namespaces, **kwargs)
 
+    @property
+    def xmlns_processing_default(self) -> str:
+        return 'stacked'
+
+    def get_xmlns_from_data(self, obj: Any) -> Optional[List[Tuple[str, str]]]:
+        return obj.xmlns if isinstance(obj, DataElement) else None
+
+    def get_namespaces(self, namespaces: Optional[NamespacesType] = None,
+                       root_only: bool = True) -> NamespacesType:
+        if not isinstance(self.source, DataElement) or self._xmlns_getter is None:
+            return super().get_namespaces(namespaces, root_only)
+
+        namespaces = get_namespace_map(namespaces)
+        for obj in self.source.iter():
+            xmlns = self.get_xmlns_from_data(obj)
+            if xmlns:
+                update_namespaces(namespaces, xmlns, obj is self.source)
+            if root_only:
+                break
+
+        return namespaces
+
     @property
     def lossy(self) -> bool:
         return False
@@ -494,24 +506,28 @@ class DataElementConverter(XMLSchemaConverter):
     def losslessly(self) -> bool:
         return True
 
-    def copy(self, **kwargs: Any) -> 'DataElementConverter':
-        obj = cast(DataElementConverter, super().copy(**kwargs))
+    def copy(self, keep_namespaces: bool = True, **kwargs: Any) -> 'DataElementConverter':
+        obj = cast(DataElementConverter, super().copy(keep_namespaces, **kwargs))
         obj.data_element_class = kwargs.get('data_element_class', self.data_element_class)
         return obj
 
-    def get_data_element(self, data: ElementData, xsd_element: 'XsdElement',
-                         xsd_type: Optional[BaseXsdType] = None) -> DataElement:
+    def get_data_element(self, data: ElementData,
+                         xsd_element: 'XsdElement',
+                         xsd_type: Optional[BaseXsdType] = None,
+                         level: int = 0) -> DataElement:
+        xmlns = self.get_effective_xmlns(data.xmlns, level, xsd_element)
         return self.data_element_class(
             tag=data.tag,
             value=data.text,
-            nsmap=self.namespaces,
+            nsmap=self._namespaces if self._use_namespaces else None,
+            xmlns=xmlns,
             xsd_element=xsd_element,
             xsd_type=xsd_type
         )
 
     def element_decode(self, data: ElementData, xsd_element: 'XsdElement',
                        xsd_type: Optional[BaseXsdType] = None, level: int = 0) -> 'DataElement':
-        data_element = self.get_data_element(data, xsd_element, xsd_type)
+        data_element = self.get_data_element(data, xsd_element, xsd_type, level)
         if self.map_attribute_names:
             data_element.attrib.update(self.map_attributes(data.attributes))
         elif data.attributes:
@@ -531,8 +547,8 @@ class DataElementConverter(XMLSchemaConverter):
 
     def element_encode(self, data_element: 'DataElement', xsd_element: 'XsdElement',
                        level: int = 0) -> ElementData:
-        self.namespaces.update(data_element.nsmap)
-        if not xsd_element.is_matching(data_element.tag, self._namespaces.get('')):
+        xmlns = self.set_context(data_element, level)
+        if not xsd_element.is_matching(data_element.tag):
             raise XMLSchemaValueError("Unmatched tag")
 
         attributes = {self.unmap_qname(k, xsd_element.attributes): v
@@ -540,7 +556,7 @@ class DataElementConverter(XMLSchemaConverter):
 
         data_len = len(data_element)
         if not data_len:
-            return ElementData(data_element.tag, data_element.value, None, attributes)
+            return ElementData(data_element.tag, data_element.value, None, attributes, xmlns)
 
         content: List[Tuple[Union[str, int], Any]] = []
         cdata_num = count(1)
@@ -552,7 +568,7 @@ class DataElementConverter(XMLSchemaConverter):
             if e.tail is not None:
                 content.append((next(cdata_num), e.tail))
 
-        return ElementData(data_element.tag, None, content, attributes)
+        return ElementData(data_element.tag, None, content, attributes, xmlns)
 
 
 class DataBindingConverter(DataElementConverter):
@@ -564,12 +580,16 @@ class DataBindingConverter(DataElementConverter):
     """
     __slots__ = ()
 
-    def get_data_element(self, data: ElementData, xsd_element: 'XsdElement',
-                         xsd_type: Optional[BaseXsdType] = None) -> DataElement:
+    def get_data_element(self, data: ElementData,
+                         xsd_element: 'XsdElement',
+                         xsd_type: Optional[BaseXsdType] = None,
+                         level: int = 0) -> DataElement:
+        xmlns = self.get_effective_xmlns(data.xmlns, level, xsd_element)
         cls = xsd_element.get_binding(self.data_element_class)
         return cls(
             tag=data.tag,
             value=data.text,
-            nsmap=self.namespaces,
+            nsmap=self._namespaces if self._use_namespaces else None,
+            xmlns=xmlns,
             xsd_type=xsd_type
         )
