@@ -14,8 +14,9 @@ import warnings
 from copy import copy as _copy
 from decimal import Decimal
 from types import GeneratorType
-from typing import TYPE_CHECKING, cast, Any, Dict, Iterator, List, Optional, Tuple, Type, Union
-from xml.etree.ElementTree import Element
+from typing import TYPE_CHECKING, cast, Any, Dict, Iterator, List, Optional, \
+    Set, Tuple, Type, Union
+from xml.etree.ElementTree import Element, ParseError
 
 from elementpath import XPath2Parser, ElementPathError, XPathContext, XPathToken, \
     ElementNode, LazyElementNode, SchemaElementNode, build_schema_node_tree
@@ -29,8 +30,10 @@ from ..aliases import ElementType, SchemaType, BaseXsdType, SchemaElementType, \
     ModelParticleType, ComponentClassType, AtomicValueType, DecodeType, \
     IterDecodeType, IterEncodeType
 from ..translation import gettext as _
-from ..helpers import get_qname, get_namespace, etree_iter_location_hints, \
-    raw_xml_encode, strictly_equal
+from ..helpers import get_qname, etree_iter_location_hints, \
+    etree_iter_namespaces, raw_xml_encode, strictly_equal
+from ..namespaces import NamespaceMapper
+from ..locations import normalize_url
 from .. import dataobjects
 from ..converters import ElementData, XMLSchemaConverter
 from ..xpath import XsdSchemaProtocol, XsdElementProtocol, XMLSchemaProxy, \
@@ -38,18 +41,18 @@ from ..xpath import XsdSchemaProtocol, XsdElementProtocol, XMLSchemaProxy, \
 from ..resources import XMLResource
 
 from .exceptions import XMLSchemaNotBuiltError, XMLSchemaValidationError, \
-    XMLSchemaTypeTableWarning
+    XMLSchemaParseError, XMLSchemaStopValidation, XMLSchemaTypeTableWarning
 from .helpers import get_xsd_derivation_attribute
 from .xsdbase import XSD_TYPE_DERIVATIONS, XSD_ELEMENT_DERIVATIONS, \
-    XsdComponent, ValidationMixin
+    XSD_VALIDATION_MODES, XsdComponent, ValidationMixin
 from .particles import ParticleMixin, OccursCalculator
 from .identities import XsdIdentity, XsdKey, XsdUnique, \
-    XsdKeyref, IdentityCounter, IdentityCounterType
+    XsdKeyref, KeyrefCounter, IdentityCounterType
 from .simple_types import XsdSimpleType
 from .attributes import XsdAttribute
 from .wildcards import XsdAnyElement
 
-if TYPE_CHECKING:  # pragma: no cover
+if TYPE_CHECKING:
     from .attributes import XsdAttributeGroup
     from .groups import XsdGroup
 
@@ -102,7 +105,8 @@ class XsdElement(XsdComponent, ParticleMixin,
     fixed: Optional[str] = None
     substitution_group: Optional[str] = None
 
-    identities: Dict[str, XsdIdentity]
+    identities: List[XsdIdentity]
+    selected_by: Set[XsdIdentity]
     alternatives = ()  # type: Union[Tuple[()], List[XsdAlternative]]
     inheritable = ()  # type: Union[Tuple[()], Dict[str, XsdAttribute]]
 
@@ -121,6 +125,7 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         if not build:
             self._build = False
+        self.selected_by = set()
         super(XsdElement, self).__init__(elem, schema, parent)
 
     def __repr__(self) -> str:
@@ -177,6 +182,7 @@ class XsdElement(XsdComponent, ParticleMixin,
                 self.substitution_group = xsd_element.substitution_group
                 self.identities = xsd_element.identities
                 self.alternatives = xsd_element.alternatives
+                self.selected_by = xsd_element.selected_by
 
             for attr_name in {'type', 'nillable', 'default', 'fixed', 'form',
                               'block', 'abstract', 'final', 'substitutionGroup'}:
@@ -298,7 +304,7 @@ class XsdElement(XsdComponent, ParticleMixin,
                 self.parse_error(msg)
 
         # Identity constraints
-        self.identities = {}
+        self.identities = []
         constraint: Union[XsdKey, XsdUnique, XsdKeyref]
         for child in self.elem:
             if child.tag == XSD_UNIQUE:
@@ -312,11 +318,11 @@ class XsdElement(XsdComponent, ParticleMixin,
                 continue
 
             if constraint.ref:
-                if constraint.name in self.identities:
+                if any(constraint.name == x.name for x in self.identities):
                     msg = _("duplicated identity constraint %r:")
                     self.parse_error(msg % constraint.name, child)
 
-                self.identities[constraint.name] = constraint
+                self.identities.append(constraint)
                 continue
 
             try:
@@ -326,7 +332,7 @@ class XsdElement(XsdComponent, ParticleMixin,
             except KeyError:
                 self.maps.identities[constraint.name] = constraint
             finally:
-                self.identities[constraint.name] = constraint
+                self.identities.append(constraint)
 
     def _parse_substitution_group(self, substitution_group: str) -> None:
         try:
@@ -416,7 +422,7 @@ class XsdElement(XsdComponent, ParticleMixin,
     def built(self) -> bool:
         return hasattr(self, 'type') and \
             (self.type.parent is None or self.type.built) and \
-            all(c.built for c in self.identities.values())
+            all(c.built for c in self.identities)
 
     @property
     def validation_attempted(self) -> str:
@@ -424,7 +430,7 @@ class XsdElement(XsdComponent, ParticleMixin,
             return 'full'
         elif self.type.validation_attempted == 'partial':
             return 'partial'
-        elif any(c.validation_attempted == 'partial' for c in self.identities.values()):
+        elif any(c.validation_attempted == 'partial' for c in self.identities):
             return 'partial'
         else:
             return 'none'
@@ -514,12 +520,12 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         if xsd_classes is None:
             yield self
-            yield from self.identities.values()
+            yield from self.identities
         else:
             if isinstance(self, xsd_classes):
                 yield self
             if issubclass(XsdIdentity, xsd_classes):
-                yield from self.identities.values()
+                yield from self.identities
 
         if self.ref is None and self.type.parent is not None:
             yield from self.type.iter_components(xsd_classes)
@@ -544,67 +550,39 @@ class XsdElement(XsdComponent, ParticleMixin,
                 return None
         return self.type.text_decode(text)
 
-    def check_dynamic_context(self, elem: ElementType, **kwargs: Any) -> None:
+    def check_dynamic_context(self, elem: ElementType,
+                              validation: str,
+                              options: Dict[str, Any]) -> Iterator[XMLSchemaValidationError]:
         try:
-            locations = kwargs['locations']
+            source: XMLResource = options['source']
         except KeyError:
             return
 
-        schema: Optional[SchemaType]
         for ns, url in etree_iter_location_hints(elem):
-            if ns not in locations:
-                locations[ns] = url
-            elif locations[ns] is None:
+            base_url = source.base_url
+            url = normalize_url(url, base_url)
+            if any(url == schema.url for schema in self.maps.iter_schemas()):
+                continue
+
+            if ns in etree_iter_namespaces(source.root, elem):
                 reason = _("schemaLocation declaration after namespace start")
-                raise XMLSchemaValidationError(self, elem, reason)
+                yield self.validation_error(validation, reason, elem, **options)
 
-            if ns == self.target_namespace:
-                schema = self.schema.include_schema(url, self.schema.base_url)
-            else:
-                schema = self.schema.import_schema(ns, url, self.schema.base_url)
-
-            if schema is None:
-                reason = _("missing dynamic loaded schema from %s") % url
-                raise XMLSchemaValidationError(self, elem, reason)
-            elif not schema.built:
-                reason = _("dynamic loaded schema change the assessment")
-                raise XMLSchemaValidationError(self, elem, reason)
-
-        if elem.attrib:
-            for name in elem.attrib:
-                if name[0] == '{':
-                    ns = get_namespace(name)
-                    if ns not in locations:
-                        locations[ns] = None
-
-        if elem.tag[0] == '{':
-            ns = get_namespace(elem.tag)
-            if ns not in locations:
-                locations[ns] = None
-
-    def start_identities(self, identities: Dict[XsdIdentity, IdentityCounter]) -> None:
-        """
-        Start tracking of XSD element's identities.
-
-        :param identities: a dictionary containing the identities counters.
-        """
-        for constraint in self.identities.values():
             try:
-                identities[constraint].clear()
-            except KeyError:
-                identities[constraint] = constraint.get_counter()
+                if ns in self.maps.namespaces:
+                    schema = self.maps.namespaces[ns][0]
+                    schema.include_schema(url)
+                    self.schema.clear()
+                    self.schema.build()
+                else:
+                    self.schema.import_schema(ns, url, base_url, build=True)
 
-    def stop_identities(self, identities: Dict[XsdIdentity, IdentityCounter]) -> None:
-        """
-        Stop tracking of XSD element's identities.
-
-        :param identities: a dictionary containing the identities counters.
-        """
-        for identity in self.identities.values():
-            try:
-                identities[identity].enabled = False
-            except KeyError:
-                identities[identity] = identity.get_counter(enabled=False)
+            except (XMLSchemaValidationError, ParseError) as err:
+                yield self.validation_error(validation, err, elem, **options)
+            except XMLSchemaParseError as err:
+                yield self.validation_error(validation, err.message, elem, **options)
+            except (OSError, IOError):
+                continue
 
     def iter_decode(self, obj: ElementType, validation: str = 'lax', **kwargs: Any) \
             -> IterDecodeType[Any]:
@@ -617,14 +595,21 @@ class XsdElement(XsdComponent, ParticleMixin,
         :return: yields a decoded object, eventually preceded by a sequence of \
         validation or decoding errors.
         """
+        error: Union[XMLSchemaValueError, XMLSchemaValidationError]
+        result: Any
+
         if self.abstract:
             reason = _("cannot use an abstract element for validation")
             yield self.validation_error(validation, reason, obj, **kwargs)
 
-        try:
-            namespaces = kwargs['namespaces']
-        except KeyError:
-            namespaces = None
+        # Control validation on element and its descendants or stop validation
+        if 'validation_hook' in kwargs:
+            value = kwargs['validation_hook'](obj, self)
+            if value:
+                if isinstance(value, str) and value in XSD_VALIDATION_MODES:
+                    validation = value
+                else:
+                    return
 
         try:
             level = kwargs['level']
@@ -636,20 +621,26 @@ class XsdElement(XsdComponent, ParticleMixin,
         except KeyError:
             identities = kwargs['identities'] = {}
 
-        self.start_identities(identities)
+        for identity in self.identities:
+            if identity in identities:
+                identities[identity].reset(obj)
+            else:
+                identities[identity] = identity.get_counter(obj)
 
         try:
             converter = kwargs['converter']
         except KeyError:
-            converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
+            converter = self._get_converter(obj, kwargs)
         else:
-            if converter is not None and not isinstance(converter, XMLSchemaConverter):
-                converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
+            if not isinstance(converter, NamespaceMapper):
+                converter = self._get_converter(obj, kwargs)
 
-        try:
-            pass  # self.check_dynamic_context(elem, **kwargs) TODO: dynamic schema load
-        except XMLSchemaValidationError as err:
-            yield self.validation_error(validation, err, obj, **kwargs)
+        if not level:
+            # Need to set base context with the right object (the resource can be lazy)
+            converter.set_context(obj, level)
+        elif kwargs.get('use_location_hints'):
+            # Use location hints for dynamic schema load
+            yield from self.check_dynamic_context(obj, validation, options=kwargs)
 
         inherited = kwargs.get('inherited')
         value = content = attributes = None
@@ -657,8 +648,10 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         # Get the instance effective type
         xsd_type = self.get_type(obj, inherited)
-        if XSI_TYPE in obj.attrib:
+        if XSI_TYPE in obj.attrib and self.schema.meta_schema is not None:
+            # Meta-schema elements ignore xsi:type (issue #350)
             type_name = obj.attrib[XSI_TYPE].strip()
+            namespaces = converter.namespaces
             try:
                 xsd_type = self.maps.get_instance_type(type_name, xsd_type, namespaces)
             except (KeyError, TypeError) as err:
@@ -666,7 +659,7 @@ class XsdElement(XsdComponent, ParticleMixin,
             else:
                 if self.identities:
                     xpath_element = XPathElement(self.name, xsd_type)
-                    for identity in self.identities.values():
+                    for identity in self.identities:
                         if isinstance(identity.elements, tuple) \
                                 or identity.selector is None:
                             continue  # Skip unbuilt or incomplete identities
@@ -681,11 +674,13 @@ class XsdElement(XsdComponent, ParticleMixin,
                         )
 
                         for e in identity.selector.token.select_results(context):
-                            if not isinstance(e, XsdElement):
+                            if not isinstance(e, (XsdElement, XsdAnyElement)):
                                 reason = _("selector xpath expression can only select elements")
                                 yield self.validation_error(validation, reason, e, **kwargs)
                             elif e not in identity.elements:
                                 identity.elements[e] = None
+                                if isinstance(e, XsdElement):
+                                    e.selected_by.add(identity)
 
             if xsd_type.is_blocked(self):
                 reason = _("usage of %r is blocked") % xsd_type
@@ -701,7 +696,6 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         # Decode attributes
         attribute_group = self.get_attributes(xsd_type)
-        result: Any
         for result in attribute_group.iter_decode(obj.attrib, validation, **kwargs):
             if isinstance(result, XMLSchemaValidationError):
                 yield self.validation_error(validation, result, obj, **kwargs)
@@ -778,7 +772,7 @@ class XsdElement(XsdComponent, ParticleMixin,
                     reason = _("must have the fixed value %r") % self.fixed
                     yield self.validation_error(validation, reason, obj, **kwargs)
 
-            elif not text and self.default is not None and kwargs.get('use_defaults'):
+            elif not text and self.default is not None and kwargs.get('use_defaults', True):
                 text = self.default
 
             if not isinstance(xsd_type, XsdSimpleType):
@@ -827,8 +821,10 @@ class XsdElement(XsdComponent, ParticleMixin,
                 if not kwargs.get('binary_types'):
                     value = str(value)
 
-        if converter is not None:
-            element_data = ElementData(obj.tag, value, content, attributes)
+        xmlns = converter.set_context(obj, level)  # Purge existing sub-contexts
+
+        if isinstance(converter, XMLSchemaConverter):
+            element_data = ElementData(obj.tag, value, content, attributes, xmlns)
             if 'element_hook' in kwargs:
                 element_data = kwargs['element_hook'](element_data, self, xsd_type)
 
@@ -837,57 +833,13 @@ class XsdElement(XsdComponent, ParticleMixin,
             except (ValueError, TypeError) as err:
                 yield self.validation_error(validation, err, obj, **kwargs)
         elif not level:
-            yield ElementData(obj.tag, value, None, attributes)
+            yield ElementData(obj.tag, value, None, attributes, None)
 
         if content is not None:
             del content
 
-        element_node: Union[None, ElementNode, LazyElementNode] = None
-
-        # Collect field values for identities that refer to this element.
-        for identity, counter in identities.items():
-            if not counter.enabled or not identity.elements:
-                continue
-            elif self in identity.elements:
-                xsd_element = self
-            elif self.ref in identity.elements:
-                xsd_element = self.ref
-            else:
-                continue
-
-            try:
-                xsd_fields: Optional[IdentityCounterType]
-                if xsd_type is self.type:
-                    xsd_fields = identity.elements[xsd_element]
-                    if xsd_fields is None:
-                        xsd_fields = identity.get_fields(xsd_element.xpath_node)
-                        identity.elements[xsd_element] = xsd_fields
-                else:
-                    xsd_element = _copy(self)
-                    xsd_element.type = xsd_type
-                    xsd_fields = identity.get_fields(xsd_element.xpath_node)
-
-                if all(x is None for x in xsd_fields):
-                    continue
-                decoders = cast(Tuple[XsdAttribute, ...], xsd_fields)
-
-                if element_node is None:
-                    try:
-                        resource = cast(XMLResource, kwargs['source'])
-                    except KeyError:
-                        element_node = LazyElementNode(obj, nsmap=_copy(namespaces))
-                    else:
-                        element_node = resource.get_xpath_node(obj)
-
-                fields = identity.get_fields(element_node, namespaces, decoders=decoders)
-            except (XMLSchemaValueError, XMLSchemaTypeError) as err:
-                yield self.validation_error(validation, err, obj, **kwargs)
-            else:
-                if any(x is not None for x in fields) or nilled:
-                    try:
-                        counter.increase(fields)
-                    except ValueError as err:
-                        yield self.validation_error(validation, err, obj, **kwargs)
+        if self.selected_by:
+            yield from self.collect_key_fields(obj, xsd_type, validation, nilled, **kwargs)
 
         # Apply non XSD optional validations
         if 'extra_validator' in kwargs:
@@ -902,14 +854,84 @@ class XsdElement(XsdComponent, ParticleMixin,
 
         # Disable collect for out of scope identities and check key references
         if 'max_depth' not in kwargs:
-            for identity in self.identities.values():
+            for identity in self.identities:
                 counter = identities[identity]
                 counter.enabled = False
                 if isinstance(identity, XsdKeyref):
+                    assert isinstance(counter, KeyrefCounter)
                     for error in counter.iter_errors(identities):
                         yield self.validation_error(validation, error, obj, **kwargs)
         elif level:
-            self.stop_identities(identities)
+            for identity in self.identities:
+                identities[identity].enabled = False
+
+    def collect_key_fields(self, elem: ElementType, xsd_type: BaseXsdType,
+                           validation: str = 'lax', nilled: bool = False,
+                           **kwargs: Any) -> Iterator[XMLSchemaValidationError]:
+        element_node: Union[ElementNode, LazyElementNode]
+        xsd_fields: Optional[IdentityCounterType]
+
+        try:
+            identities = kwargs['identities']
+            resource = cast(XMLResource, kwargs['source'])
+        except KeyError:
+            # skip identities collect if identity map or XML source are missing
+            return
+
+        try:
+            namespaces = kwargs['namespaces']
+        except KeyError:
+            namespaces = None
+
+        element_node = resource.get_xpath_node(elem)
+
+        xsd_element = self if self.ref is None else self.ref
+        if xsd_element.type is not xsd_type:
+            xsd_element = _copy(xsd_element)
+            xsd_element.type = xsd_type
+
+        # Collect field values for identities that refer to this element.
+        for identity in self.selected_by:
+            try:
+                counter = identities[identity]
+            except KeyError:
+                continue
+            else:
+                if not counter.enabled or not identity.elements:
+                    continue
+
+            if counter.elements is None:
+                # Apply selector on Element ancestor for obtain the selected elements
+                root_node = resource.get_xpath_node(counter.elem)
+                context = XPathContext(root_node)
+                assert identity.selector is not None and identity.selector.token is not None
+                counter.elements = set(identity.selector.token.select_results(context))
+
+            if elem not in counter.elements:
+                continue
+
+            try:
+                if xsd_element.type is self.type and xsd_element in identity.elements:
+                    xsd_fields = identity.elements[xsd_element]
+                    if xsd_fields is None:
+                        xsd_fields = identity.get_fields(xsd_element.xpath_node)
+                        identity.elements[xsd_element] = xsd_fields
+                else:
+                    xsd_fields = identity.get_fields(xsd_element.xpath_node)
+
+                if all(x is None for x in xsd_fields):
+                    continue
+
+                decoders = cast(Tuple[XsdAttribute, ...], xsd_fields)
+                fields = identity.get_fields(element_node, namespaces, decoders=decoders)
+            except (XMLSchemaValueError, XMLSchemaTypeError) as err:
+                yield self.validation_error(validation, err, elem, **kwargs)
+            else:
+                if any(x is not None for x in fields) or nilled:
+                    try:
+                        counter.increase(fields)
+                    except ValueError as err:
+                        yield self.validation_error(validation, err, elem, **kwargs)
 
     def to_objects(self, obj: ElementType, with_bindings: bool = False, **kwargs: Any) \
             -> DecodeType['dataobjects.DataElement']:
@@ -944,37 +966,33 @@ class XsdElement(XsdComponent, ParticleMixin,
         try:
             converter = kwargs['converter']
         except KeyError:
-            converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
+            converter = self._get_converter(obj, kwargs)
         else:
             if not isinstance(converter, XMLSchemaConverter):
-                converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
+                converter = self._get_converter(obj, kwargs)
 
         try:
             level = kwargs['level']
         except KeyError:
-            level = 0
+            level = kwargs['level'] = 0
 
         try:
             element_data = converter.element_encode(obj, self, level)
         except (ValueError, TypeError) as err:
             yield self.validation_error(validation, err, obj, **kwargs)
             return
-        else:
-            if not level:
-                if not self.is_matching(element_data.tag, self.default_namespace):
-                    errors.append("data tag does not match XSD element name")
 
-                if 'max_depth' in kwargs and kwargs['max_depth'] == 0:
-                    for e in errors:
-                        yield self.validation_error(validation, e, **kwargs)
-                    return
+        if 'max_depth' in kwargs and kwargs['max_depth'] == 0 and not level:
+            for e in errors:
+                yield self.validation_error(validation, e, **kwargs)
+            return
 
         text = None
         children = element_data.content
         attributes = ()
 
         xsd_type = self.get_type(element_data)
-        if XSI_TYPE in element_data.attributes:
+        if XSI_TYPE in element_data.attributes and self.schema.meta_schema is not None:
             type_name = element_data.attributes[XSI_TYPE].strip()
             try:
                 xsd_type = self.maps.get_instance_type(type_name, xsd_type, converter)
@@ -1037,7 +1055,7 @@ class XsdElement(XsdComponent, ParticleMixin,
 
             elif self.fixed is not None:
                 text = self.fixed
-            elif self.default is not None and kwargs.get('use_defaults'):
+            elif self.default is not None and kwargs.get('use_defaults', True):
                 text = self.default
 
         elif xsd_type.has_simple_content():
@@ -1051,7 +1069,7 @@ class XsdElement(XsdComponent, ParticleMixin,
 
             elif self.fixed is not None:
                 text = self.fixed
-            elif self.default is not None and kwargs.get('use_defaults'):
+            elif self.default is not None and kwargs.get('use_defaults', True):
                 text = self.default
 
         else:
@@ -1074,29 +1092,24 @@ class XsdElement(XsdComponent, ParticleMixin,
         if not name:
             return False
         elif default_namespace and name[0] != '{':
-            qname = f'{{{default_namespace}}}{name}'
-            if name == self.name or qname == self.name:
-                return True
-            return any(name == e.name or qname == e.name for e in self.iter_substitutes())
-        elif name == self.name:
-            return True
-        else:
-            return any(name == e.name for e in self.iter_substitutes())
+            name = f'{{{default_namespace}}}{name}'
+
+            # Workaround for backward compatibility of XPath selectors on schemas.
+            # It could be removed in v3.0.
+            if not self.qualified and default_namespace == self.target_namespace:
+                return (name == self.qualified_name or
+                        any(name == e.qualified_name for e in self.iter_substitutes()))
+
+        return name == self.name or any(name == e.name for e in self.iter_substitutes())
 
     def match(self, name: Optional[str], default_namespace: Optional[str] = None,
               **kwargs: Any) -> Optional['XsdElement']:
         if not name:
             return None
         elif default_namespace and name[0] != '{':
-            qname = f'{{{default_namespace}}}{name}'
-            if name == self.name or qname == self.name:
-                return self
+            name = f'{{{default_namespace}}}{name}'
 
-            for xsd_element in self.iter_substitutes():
-                if name == xsd_element.name or qname == xsd_element.name:
-                    return xsd_element
-
-        elif name == self.name:
+        if name == self.name:
             return self
         else:
             for xsd_element in self.iter_substitutes():
@@ -1284,7 +1297,7 @@ class Xsd11Element(XsdElement):
     @property
     def built(self) -> bool:
         return (self.type.parent is None or self.type.built) and \
-            all(c.built for c in self.identities.values()) and \
+            all(c.built for c in self.identities) and \
             all(a.built for a in self.alternatives)
 
     @property
@@ -1299,12 +1312,12 @@ class Xsd11Element(XsdElement):
     def iter_components(self, xsd_classes: ComponentClassType = None) -> Iterator[XsdComponent]:
         if xsd_classes is None:
             yield self
-            yield from self.identities.values()
+            yield from self.identities
         else:
             if isinstance(self, xsd_classes):
                 yield self
 
-            for obj in self.identities.values():
+            for obj in self.identities:
                 if isinstance(obj, xsd_classes):
                     yield obj
 
@@ -1403,6 +1416,49 @@ class Xsd11Element(XsdElement):
             warnings.warn(msg.format(e1, e2), XMLSchemaTypeTableWarning, stacklevel=3)
         return True
 
+    def check_dynamic_context(self, elem: ElementType,
+                              validation: str,
+                              options: Dict[str, Any]) -> Iterator[XMLSchemaValidationError]:
+        try:
+            source = options['source']
+        except KeyError:
+            return
+
+        for ns, url in etree_iter_location_hints(elem):
+            base_url = source.base_url
+            url = normalize_url(url, base_url)
+            if any(url == schema.url for schema in self.maps.iter_schemas()):
+                continue
+
+            try:
+                if ns in self.maps.namespaces:
+                    schema = self.maps.namespaces[ns][0]
+                    schema.include_schema(url)
+                    schema.clear()
+                    schema.build()
+                else:
+                    schema = self.schema
+                    schema.import_schema(ns, url, base_url, build=True)
+
+                def stop_validation(e: ElementType, _xsd_element: XsdElement) -> bool:
+                    if e is elem:
+                        raise XMLSchemaStopValidation()
+                    return False
+
+                errors = list(schema.iter_errors(source, validation_hook=stop_validation))
+                if len(options['errors']) != len(errors) or \
+                        any(e1.elem is not e2.elem for e1, e2 in zip(options['errors'], errors)):
+                    reason = _(f"adding schema at {url} change the "
+                               f"assessment outcome of previous items")
+                    yield self.validation_error(validation, reason, elem, **options)
+
+            except (XMLSchemaValidationError, ParseError) as err:
+                yield self.validation_error(validation, err, elem, **options)
+            except XMLSchemaParseError as err:
+                yield self.validation_error(validation, err.message, elem, **options)
+            except (OSError, IOError):
+                continue
+
 
 class XsdAlternative(XsdComponent):
     """
