@@ -166,8 +166,7 @@ class XsdWildcard(XsdComponent):
         elif not default_namespace:
             return self.is_namespace_allowed('')
         else:
-            return self.is_namespace_allowed('') or \
-                self.is_namespace_allowed(default_namespace)
+            return self.is_namespace_allowed(default_namespace)
 
     def is_namespace_allowed(self, namespace: str) -> bool:
         if self.not_namespace:
@@ -525,7 +524,11 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
 
         if validation != 'skip' and self.process_contents == 'strict':
             yield self.validation_error(validation, reason, obj, **kwargs)
-        yield from self.any_type.iter_decode(obj, validation, **kwargs)
+
+        xsd_element = self.maps.validator.create_element(
+            obj.tag, parent=self, form='unqualified'
+        )
+        yield from xsd_element.iter_decode(obj, validation, **kwargs)
 
     def iter_encode(self, obj: Tuple[str, ElementType], validation: str = 'lax', **kwargs: Any) \
             -> IterEncodeType[Any]:
@@ -565,12 +568,12 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
         try:
             converter = kwargs['converter']
         except KeyError:
-            converter = kwargs['converter'] = self.schema.get_converter(**kwargs)
+            converter = self._get_converter(value, kwargs)
 
         try:
             level = kwargs['level']
         except KeyError:
-            level = 0
+            level = kwargs['level'] = 0
 
         try:
             element_data = converter.element_encode(value, xsd_element, level)
@@ -584,6 +587,7 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
 
         if validation != 'skip' and self.process_contents == 'strict':
             yield self.validation_error(validation, reason, **kwargs)
+
         yield from self.any_type.iter_encode(obj, validation, **kwargs)
 
     def is_overlap(self, other: ModelParticleType) -> bool:
@@ -755,10 +759,10 @@ class Xsd11AnyElement(XsdAnyElement):
                     **kwargs: Any) -> bool:
         """
         Returns `True` if the component name is matching the name provided as argument,
-        `False` otherwise. For XSD elements the matching is extended to substitutes.
+        `False` otherwise.
 
         :param name: a local or fully-qualified name.
-        :param default_namespace: used if it's not None and not empty for completing \
+        :param default_namespace: used by the XPath processor for completing \
         the name argument in case it's a local name.
         :param group: used only by XSD 1.1 any element wildcards to verify siblings in \
         case of ##definedSibling value in notQName attribute.
@@ -774,8 +778,7 @@ class Xsd11AnyElement(XsdAnyElement):
                 return False
         else:
             name = f'{{{default_namespace}}}{name}'
-            if not self.is_namespace_allowed('') \
-                    and not self.is_namespace_allowed(default_namespace):
+            if not self.is_namespace_allowed(default_namespace):
                 return False
 
         if group in self.precedences:
