@@ -10,10 +10,9 @@
 """
 This module defines a proxy class and a mixin class for enabling XPath on schemas.
 """
-import sys
 from abc import abstractmethod
 from typing import cast, overload, Any, Dict, Iterator, List, Optional, \
-    Sequence, Set, TypeVar, Union
+    Sequence, Set, TypeVar, Union, runtime_checkable, Protocol
 import re
 
 from elementpath import XPath2Parser, XPathSchemaContext, \
@@ -21,23 +20,17 @@ from elementpath import XPath2Parser, XPathSchemaContext, \
 
 from .exceptions import XMLSchemaValueError, XMLSchemaTypeError
 from .names import XSD_NAMESPACE
-from .aliases import NamespacesType, SchemaType, BaseXsdType, XPathElementType
+from .aliases import NamespacesType, SchemaType, BaseXsdType
 from .helpers import get_qname, local_name, get_prefixed_qname
 
-if sys.version_info < (3, 8):  # pragma: no cover
-    XsdSchemaProtocol = SchemaType
-    XsdElementProtocol = XPathElementType
-    XsdTypeProtocol = BaseXsdType
-else:
-    from typing import runtime_checkable, Protocol
+XsdTypeProtocol = protocols.XsdTypeProtocol
+XsdSchemaProtocol = protocols.XsdSchemaProtocol
 
-    XsdTypeProtocol = protocols.XsdTypeProtocol
-    XsdSchemaProtocol = protocols.XsdSchemaProtocol
 
-    @runtime_checkable
-    class XsdElementProtocol(protocols.XsdElementProtocol, Protocol):
-        schema: XsdSchemaProtocol
-        attributes: Dict[str, Any]
+@runtime_checkable
+class XsdElementProtocol(protocols.XsdElementProtocol, Protocol):
+    schema: XsdSchemaProtocol
+    attributes: Dict[str, Any]
 
 
 _REGEX_TAG_POSITION = re.compile(r'\b\[\d+]')
@@ -66,7 +59,7 @@ class XMLSchemaProxy(AbstractSchemaProxy):
 
     def bind_parser(self, parser: XPath2Parser) -> None:
         parser.schema = self
-        parser.symbol_table = dict(parser.__class__.symbol_table)
+        parser.symbol_table = dict(parser.__class__.symbol_table)  # type: ignore[arg-type]
 
         with self._schema.lock:
             if self._schema.xpath_tokens is None:
@@ -188,14 +181,11 @@ class ElementPathMixin(Sequence[E]):
         :param namespaces: an optional map from namespace prefix to namespace URI. \
         If this argument is not provided the schema's namespaces are used.
         """
-        if namespaces is None:
-            namespaces = {k: v for k, v in self.namespaces.items() if k}
-            namespaces[''] = self.xpath_default_namespace
-        elif '' not in namespaces:
-            namespaces[''] = self.xpath_default_namespace
-
         xpath_namespaces: Dict[str, str] = XPath2Parser.DEFAULT_NAMESPACES.copy()
-        xpath_namespaces.update(namespaces)
+        if namespaces is None:
+            xpath_namespaces.update(self.namespaces)
+        else:
+            xpath_namespaces.update(namespaces)
         return xpath_namespaces
 
     def is_matching(self, name: Optional[str], default_namespace: Optional[str] = None) -> bool:
